@@ -2,18 +2,19 @@
 
 This module provides functions used to compute synchrony metrics on multivariate time series. It contains the following functions:
 
-    * recurrence_matrix - Creates a recurrence matrix from a multivariate time series. 
-    * get_diagonal_lengths - Finds the lengths of diagonals in a recurrence matrix. Used by rqa_metrics. 
-    * rqa_metrics - Computes the proportion of recurrence, proportion of determinism, average diagonal length and longest diagonal length for an input recurrence matrix. 
-    * rho - A cluster-phase synchrony metric.
-    * coherence_team - A synchrony metric based on spectral density.
-    * convert_to_terciles - Takes a time series and returns a time series where each value is replaced by a number indicating which tercile it belongs in. Used by pattern_entropy. 
-    * symbolic_entropy - A metric based on the entropy of the combined 'state' across a multivariate time series. 
-    * kuramoto_weak_null - Tests the significance of the observed Kuramoto order parameter values in a sample of multivariate time series. 
-    * metric_fixed_parameters - Provides a copy of a function to calculate a synchrony metric, but with all parameters fixed except the input data. For use with apply_windowed when functions have multiple parameters. 
-    * apply_windowed - A function used to apply other functions in a windowed fashion. 
-    * shuffle_recordings - Creates surrogate_data by shuffling variables between time series in a sample of multivariate time series. 
-    * shuffle_time_windows - Creates surrogate_data by shuffling time windows, separately for each variable of a multivariate time series. 
+ * recurrence_matrix - Creates a recurrence matrix from a multivariate time series. 
+ * get_diagonal_lengths - Finds the lengths of diagonals in a recurrence matrix. Used by rqa_metrics. 
+ * rqa_metrics - Computes the proportion of recurrence, proportion of determinism, average diagonal length and longest diagonal length for an input recurrence matrix. 
+ * rho - A cluster-phase synchrony metric.
+ * coherence_team - A synchrony metric based on spectral density.
+ * sum_normalized_csd - A synchrony metric based on cross-spectral density, similar to coherence_team().
+ * convert_to_terciles - Takes a time series and returns a time series where each value is replaced by a number indicating which tercile it belongs in. Used by pattern_entropy. 
+ * symbolic_entropy - A metric based on the entropy of the combined 'state' across a multivariate time series. 
+ * kuramoto_weak_null - Tests the significance of the observed Kuramoto order parameter values in a sample of multivariate time series. 
+ * metric_fixed_parameters - Provides a copy of a function to calculate a synchrony metric, but with all parameters fixed except the input data. For use with apply_windowed when functions have multiple parameters. 
+ * apply_windowed - A function used to apply other functions in a windowed fashion. 
+ * shuffle_recordings - Creates surrogate_data by shuffling variables between time series in a sample of multivariate time series. 
+ * shuffle_time_windows - Creates surrogate_data by shuffling time windows, separately for each variable of a multivariate time series. 
 """
 
 import numpy as np
@@ -156,9 +157,9 @@ def rqa_metrics(recurrence_matrix, min_length=2):
                 det += length * count
 
             mean_length = rec
-
+            
+        det = det / rec ##Divide by the number of recurrent points (before rec becomes a fraction)
         rec = rec / (recurrence_matrix.shape[0]*(recurrence_matrix.shape[1]-1)/2) ##The number of off-diagonal cells in the upper triangle 
-        det = det / (recurrence_matrix.shape[0]*(recurrence_matrix.shape[1]-1)/2) ##The number of off-diagonal cells in the upper triangle
         mean_length = mean_length / sum(diagonal_length_counts.values())
         max_length = max(diagonal_length_counts.keys())
     
@@ -230,6 +231,44 @@ def coherence_team(data, nperseg=None):
     return np.mean(coherence_scores)
 
 
+def sum_normalized_csd(data):
+    """Returns a quantity, based on the cross-spectral density (CSD), similar to that of coherence_team() but which is less impacted by Gaussian noise.
+    
+    Parameters
+    ----------
+    data: ndarray
+        An array containing the time series of measurements with shape (number_signals, duration).
+    
+    Returns
+    -------
+    aggregated_csd: float
+        The sum-normalized CSD quantity.
+    """
+    
+    ## Set nperseg to a reasonable value for shorter input lengths
+    if data.shape[1] // 256 < 4: ## Default value is 256
+    
+        nperseg = data.shape[1] // 4
+    
+    else:
+    
+        nperseg = None ## Let scipy set it to default value
+    
+    csd_scores = []
+    
+    for i, x in enumerate(data):
+    
+        for j, y in enumerate(data):
+        
+            if i < j:
+            
+                csd_scores.append(
+                    (np.abs(scipy.signal.csd(x, y, nperseg=nperseg)[1]) ** 2).sum() / (scipy.signal.csd(x, x, nperseg=nperseg)[1] * scipy.signal.csd(y, y, nperseg=nperseg)[1]).sum()
+                )
+    
+    return np.mean(csd_scores)
+
+
 def convert_to_terciles(data):
     """Maps the input time series to numbers representing 'low', 'medium' and 'high' values. The thresholds for deciding 'low', 'medium' and 'high' are terciles. 
     
